--SM64 Lua script for BizHawk emulator
--Prints Mario, camera, terrain to screen
local hp
local coins
local stars
local level
local act
local area
local surface
local mario_pos
local camera_pos
local terrain
local hspeed
local vspeed

-- see console for other memory domains
console.write(memory.getmemorydomainlist())
console.write(memory.usememorydomain("RDRAM"))

-- table of terrain names. probably a better way to init this
local terrain_table = {}
terrain_table[0x00] = "normal"
terrain_table[0x01] = "lethal_lava"
terrain_table[0x05] = "hang"
terrain_table[0x0A] = "deathfloor"
terrain_table[0x0E] = "water_currents"
terrain_table[0x12] = "void"
terrain_table[0x13] = "very_slippery"
terrain_table[0x14] = "slippery"
terrain_table[0x15] = "climbable"
terrain_table[0x28] = "wall"
terrain_table[0x29] = "grass"
terrain_table[0x2A] = "unclimbable"
terrain_table[0x2C] = "windy"
terrain_table[0x2E] = "icy"
terrain_table[0x30] = "flat"
terrain_table[0x36] = "snowy"
terrain_table[0x37] = "snowy2"
terrain_table[0x76] = "fence"
terrain_table[0x7B] = "vanishing_wall"
terrain_table[0xFD] = "pool_warp"

-- terrain type to string description
function terrain2str(terrain)
   if (0x1B <= terrain and terrain <= 0x1E) then
      return string.format("switch%02X", terrain)
   elseif (0xA6 <= terrain and terrain <= 0xCF) then
      return string.format("paintingf%02X", terrain)
   elseif (0xD3 <= terrain and terrain <= 0xF8) then
      return string.format("paintingb%02X", terrain)
   elseif terrain_table[terrain] ~= nil then
      return terrain_table[terrain]
   else
      return "unknown"
   end
end

-- print float triple from memory
function read3float(base)
   arr = {}
   for i=0,2 do
      arr[i] = memory.readfloat(base + 4*i, true)
   end
   return arr
end

-- print float triple
function print3float(x, y, name, pos)
  gui.text(x, y, string.format("%s %5.1f, %5.1f, %5.1f", name, pos[2], pos[0], pos[1]))
end

-- colors for HP display
local hp_colors = {"red", "red", "yellow", "yellow", "lightgreen", "lightgreen", "lightblue", "lightblue"}

local vspace =  14 -- spacing between each line
local vstart = 400 -- y value to start printing

while true do
  hp    = memory.readbyte(0x033B21E)
  coins = memory.read_u16_be(0x033B218)
  stars = memory.read_u16_be(0x033B21A)
  level = memory.readbyte(0x033B249)
  area  = memory.readbyte(0x033B24A)
  act   = memory.readbyte(0x0331620)
  mario_pos = read3float(0x033B1AC)
  camera_pos = read3float(0x033C6A4)
  terrain = memory.read_u16_be(0x033C5F0)
  hspeed = memory.readfloat(0x033B1C4, true)
  vspeed = memory.readfloat(0x033B1BC, true)
  gui.text(  0, 0, "HP: " .. hp, hp_colors[hp])
  gui.text( 80, 0, "Coins: " .. coins)
  gui.text(200, 0, "Stars: " .. stars)
  gui.text(320, 0, "Level: " .. level)
  gui.text(420, 0, "Area: " .. area)
  gui.text(520, 0, "Act: " .. act)
  gui.text(  0, vstart+0*vspace, string.format("Hspeed: %5.1f", hspeed))
  gui.text(150, vstart+0*vspace, string.format("Vspeed: %5.1f", vspeed))
  print3float(0, vstart+1*vspace, "Mario: ", mario_pos)
  print3float(0, vstart+2*vspace, "Camera:", camera_pos)
  gui.text(0,  vstart+3*vspace, string.format("Terrain: %02X %s", terrain, terrain2str(terrain)))
  emu.frameadvance()
end
