--SM64 Lua script for BizHawk emulator

local OBJ_BASE  = 0x033D488 -- base address of objects array
local OBJ_LEN   = 0x260      -- length of each object in bytes
local koopa_idx = -1

console.clear()
-- specify RDRAM memory domain
memory.usememorydomain("RDRAM")

-- read float triple from memory
function read3float(base)
   arr = {}
   for i=0,2 do
      arr[i] = memory.readfloat(base + 4*i, true)
   end
   return arr
end

-- parse object data from address base
function ReadObject(base)
   local obj = {}
   obj.scale = read3float(base + 0x2C)
   obj.animFrame = memory.read_s16_be(base + 0x40)
   obj.active = memory.read_s32_be(base + 0x74)
   obj.position = read3float(base + 0xA0)
   obj.rotation = read3float(base + 0xD0)
   obj.rotation2 = read3float(base + 0xDC)
   obj.scaleXYZ = memory.readfloat(base + 0xF8, true)
   obj.action1 = memory.read_s32_be(base + 0x124)
   obj.action2 = memory.read_s32_be(base + 0x144)
   obj.currentAction = memory.read_s32_be(base + 0x14C)
   obj.action3 = memory.read_s32_be(base + 0x150)
   obj.timer = memory.read_s32_be(base + 0x154)
   obj.distance = memory.readfloat(base + 0x15C, true)
   obj.appear = memory.readfloat(base + 0x19C, true)
   obj.collision = {memory.readfloat(base + 0x1F8, true), memory.readfloat(base + 0x1FC, true)}
   obj.behavior = memory.read_u32_be(base + 0x20C)
   return obj
end

-- find object by behavior
-- obj_beh is offset in segment 0x13, behaviors (e.g. 0x01F4 is King Bob-Omb)
function FindByBehavior(obj_beh)
   for idx=0,256 do
      beh = memory.read_u32_be(OBJ_BASE + OBJ_LEN * idx + 0x20C)
      beh = beh - 0x800EB180
      if beh == obj_beh then
         return idx
      end
   end
   return -1
end

-- search the objects for Koopa and update idx if found
function FindKoopa()
   koopa_idx = FindByBehavior(0x4580)
   console.log("Koopa index: " .. koopa_idx)
   if (koopa_idx >= 0) then
      koopa = ReadObject(OBJ_BASE + OBJ_LEN * koopa_idx)
      forms.settext(scalex, koopa.scale[0])
      forms.settext(scaley, koopa.scale[1])
      forms.settext(scalez, koopa.scale[2])
   end
end

-- update X,Y,Z scaling fields in koopa object struct
function UpdateKoopa()
   if (koopa_idx >= 0) then
      -- TODO: error checking?
      x = tonumber(forms.gettext(scalex))
      y = tonumber(forms.gettext(scaley))
      z = tonumber(forms.gettext(scalez))
      memory.writefloat(OBJ_BASE + OBJ_LEN * koopa_idx + 0x02C, x, true)
      memory.writefloat(OBJ_BASE + OBJ_LEN * koopa_idx + 0x030, y, true)
      memory.writefloat(OBJ_BASE + OBJ_LEN * koopa_idx + 0x034, z, true)
   end
end

-- create user form and widgets
form = forms.newform(80, 160, "SM64 Objects")
koopa_button = forms.button(form, "Find Koopa", FindKoopa, 0, 0)
scalex = forms.textbox(form, "-", 100, 20, null, 0, 24)
scaley = forms.textbox(form, "-", 100, 20, null, 0, 48)
scalez = forms.textbox(form, "-", 100, 20, null, 0, 72)
set_button = forms.button(form, "Assign", UpdateKoopa, 0, 96)

while true do
   -- TODO: update fields in here if the game changes anything
   emu.frameadvance()
end
