--SM64 Lua script for BizHawk emulator
--Prints objects to screen
--Controls: 
-- * DPad Right/Left: next/prev object
-- * DPad Down/Up: cycle through hex display: 0x000, 0x130, Off
console.write(memory.usememorydomain("RDRAM"))

-- table of behavior names. index is offset into segment 0x13
local behaviors = {}
behaviors[0x0000] = "star_door"
behaviors[0x0054] = "mr_i"
behaviors[0x008C] = "mr_i_body"
behaviors[0x00AC] = "mr_i_particle"
behaviors[0x00F8] = "giant_piranha_particle"
behaviors[0x0118] = "giant_pole"
behaviors[0x0144] = "pole_grabbing"
behaviors[0x0174] = "thi_top_trap"
behaviors[0x0194] = "thi_tiny_top"
behaviors[0x01AC] = "cap_switch_collision_child"
behaviors[0x01CC] = "activate_cap_switch"
behaviors[0x01F4] = "king_bobomb"
behaviors[0x0254] = "bobomb_anchor_mario"
behaviors[0x0278] = "beta_chest"
behaviors[0x029C] = "beta_chest_upper"
behaviors[0x02B8] = "bubble_mario"
behaviors[0x02E4] = "bubble_maybe"
behaviors[0x0338] = "small_water_wave"
behaviors[0x0398] = "small_water_wave_398"
behaviors[0x03BC] = "water_air_bubble"
behaviors[0x0400] = "small_particle"
behaviors[0x0428] = "water_waves"
behaviors[0x0444] = "small_particle_snow"
behaviors[0x046C] = "small_particle_bubbles"
behaviors[0x0494] = "fish_group"
behaviors[0x04A8] = "opened_cannon"
behaviors[0x04E4] = "cannon_barrel"
behaviors[0x0500] = "cannon_base"
behaviors[0x0528] = "chuckya"
behaviors[0x0584] = "chuckya_anchor_mario"
behaviors[0x05A8] = "unused_05A8"
behaviors[0x05B4] = "rotating_platform"
behaviors[0x05D8] = "tower"
behaviors[0x0600] = "bullet_bill_cannon"
behaviors[0x0624] = "breakable_wall"
behaviors[0x0638] = "breakable_wall_2"
behaviors[0x066C] = "kickable_board"
behaviors[0x06A4] = "tower_door"
behaviors[0x06D8] = "rotating_counter_clockwise"
behaviors[0x06E0] = "clocklike_rotation"
behaviors[0x0708] = "koopa_shell_underwater"
behaviors[0x0720] = "exit_podium_warp"
behaviors[0x075C] = "fading_warp"
behaviors[0x0780] = "warp"
behaviors[0x07A0] = "warp_pipe"
behaviors[0x07DC] = "white_puff_explosion"
behaviors[0x07F8] = "spawned_star"
behaviors[0x080C] = "unused_080C"
behaviors[0x0830] = "mr_i_blue_coin"
behaviors[0x0888] = "coin_inside_boo"
behaviors[0x08D0] = "coin_formation_spawn"
behaviors[0x08EC] = "coin_formation"
behaviors[0x090C] = "one_coin"
behaviors[0x091C] = "collectable_coin"
behaviors[0x0940] = "temporary_coin"
behaviors[0x0964] = "three_coins_spawn"
behaviors[0x0984] = "ten_coins_spawn"
behaviors[0x09A4] = "single_coin_gets_spawned"
behaviors[0x09E0] = "coin_sparkles"
behaviors[0x0A14] = "golden_coin_sparkles"
behaviors[0x0A34] = "wall_tiny_star_particle"
behaviors[0x0A54] = "wall_tiny_star_particle_spawn"
behaviors[0x0A78] = "pound_tiny_star_particle"
behaviors[0x0A98] = "pound_tiny_star_particle_spawn"
behaviors[0x0ABC] = "punch_tiny_triangle"
behaviors[0x0AD8] = "punch_tiny_triangle_spawn"
behaviors[0x0AFC] = "door_warp"
behaviors[0x0B0C] = "door"
behaviors[0x0B58] = "grindel"
behaviors[0x0B8C] = "thwomp_2"
behaviors[0x0BC8] = "thwomp"
behaviors[0x0C04] = "tumbling_bridge_platform"
behaviors[0x0C28] = "wf_tumbling_bridge"
behaviors[0x0C44] = "bbh_tumbling_platform_group"
behaviors[0x0C64] = "tumbling_platform"
behaviors[0x0C84] = "burning"
behaviors[0x0CC8] = "another_elavator"
behaviors[0x0CFC] = "rr_elevator_platform"
behaviors[0x0D30] = "hmc_elevator_platform"
behaviors[0x0D6C] = "water_mist"
behaviors[0x0D98] = "water_mist_spawn"
behaviors[0x0DB4] = "break_box_triangle"
behaviors[0x0DD8] = "water_mist2"
behaviors[0x0DFC] = "unused_0DFC"
behaviors[0x0E24] = "pound_white_puffs"
behaviors[0x0E3C] = "ground_sand"
behaviors[0x0E40] = "unused_0E40"
behaviors[0x0E58] = "ground_snow"
behaviors[0x0E70] = "wind"
behaviors[0x0E88] = "end_toad"
behaviors[0x0EAC] = "end_peach"
behaviors[0x0ED0] = "piranha_particles_spawn"
behaviors[0x0F08] = "ukiki"
behaviors[0x0F14] = "unused_0F14"
behaviors[0x0F2C] = "little_cage2"
behaviors[0x0F48] = "little_cage"
behaviors[0x0F9C] = "bifs_sinking_platforms"
behaviors[0x0FC8] = "bifs_sinking_cage_platform"
behaviors[0x1000] = "ddd_moving_pole"
behaviors[0x1030] = "tilting_inverted_pyramid"
behaviors[0x1064] = "squishable_platform"
behaviors[0x1098] = "cut_out_object"
behaviors[0x10A8] = "beta_moving_flames_spawn"
behaviors[0x10B8] = "beta_moving_flames"
behaviors[0x10D8] = "rr_rotating_platform_with_fire"
behaviors[0x1108] = "flamethrower"
behaviors[0x1124] = "flamethrower_flame"
behaviors[0x1168] = "bouncing_fireball"
behaviors[0x1184] = "bouncing_fireball_flame"
behaviors[0x11D0] = "bowser_shock_wave"
behaviors[0x11EC] = "flame_mario"
behaviors[0x1214] = "black_smoke_mario"
behaviors[0x1254] = "black_smoke_bowser"
behaviors[0x127C] = "black_smoke_upward"
behaviors[0x1298] = "multiple_coins"
behaviors[0x12B4] = "spindrift"
behaviors[0x12F4] = "tower_platform_group"
behaviors[0x1318] = "wf_sliding_platform"
behaviors[0x1340] = "wf_elevator_platform"
behaviors[0x1368] = "wf_solid_platform"
behaviors[0x1390] = "snow_leaf_particle_spawn"
behaviors[0x13A8] = "tree_snow"
behaviors[0x13C4] = "tree_leaf"
behaviors[0x13DC] = "another_tilting_platform"
behaviors[0x1408] = "squarish_path_moving"
behaviors[0x142C] = "piranha_plant_bubble"
behaviors[0x1448] = "piranha_plant_waking_bubbles"
behaviors[0x1468] = "purple_switch_staircase"
behaviors[0x1478] = "floor_switch_grills"
behaviors[0x1484] = "floor_switch_hardcoded_model"
behaviors[0x14AC] = "floor_switch_hidden_objects"
behaviors[0x14BC] = "hidden_object"
behaviors[0x14E0] = "breakable_box"
behaviors[0x1518] = "pushable"
behaviors[0x1548] = "heave_ho"
behaviors[0x15A4] = "heave_ho_throw_mario"
behaviors[0x15C0] = "ccm_touched_star_spawn"
behaviors[0x15E4] = "pound_explodes"
behaviors[0x1608] = "beta_trampoline"
behaviors[0x1634] = "beta_trampoline_spawn"
behaviors[0x1650] = "jumping_box"
behaviors[0x167C] = "boo_cage"
behaviors[0x16AC] = "stub"
behaviors[0x16B8] = "igloo"
behaviors[0x16E4] = "bowser_key_2"
behaviors[0x1714] = "grand_star"
behaviors[0x1744] = "beta_boo_key_inside"
behaviors[0x1778] = "beta_boo_key_outside"
behaviors[0x179C] = "bullet_bill"
behaviors[0x17F4] = "white_puff_smoke"
behaviors[0x1820] = "unused_1820"
behaviors[0x1828] = "bowser_tail_anchor"
behaviors[0x1850] = "bowser"
behaviors[0x18CC] = "bowser_body_anchor"
behaviors[0x1904] = "bowser_flame_spawn"
behaviors[0x1920] = "tilting_bowser_lava_platform"
behaviors[0x1958] = "falling_bowser_platform"
behaviors[0x1984] = "blue_bowser_flame"
behaviors[0x19C8] = "flame_floating_landing"
behaviors[0x1A0C] = "blue_flames_group"
behaviors[0x1A30] = "flame_bouncing"
behaviors[0x1A74] = "flame_moving_forward_growing"
behaviors[0x1AA4] = "flame_bowser"
behaviors[0x1AE8] = "flame_large_burning_out"
behaviors[0x1B2C] = "blue_fish"
behaviors[0x1B54] = "tank_fish_groups"
behaviors[0x1B70] = "checkerboard_elevator_group"
behaviors[0x1B88] = "checkerboard_platform_sub"
behaviors[0x1BB4] = "door_key1"
behaviors[0x1BD4] = "door_key2"
behaviors[0x1BF4] = "invisible_objects_under_bridge"
behaviors[0x1C04] = "water_level_pillar"
behaviors[0x1C34] = "ddd_warp"
behaviors[0x1C58] = "moat_grills"
behaviors[0x1C7C] = "clock_big_arm"
behaviors[0x1C8C] = "rotating_small_clock_arm"
behaviors[0x1CB0] = "ukiki_open_cage"
behaviors[0x1D0C] = "stub_1D0C"
behaviors[0x1D14] = "lll_rotating_ccw"
behaviors[0x1D40] = "sinks_when_stepped_on"
behaviors[0x1D70] = "stub_1D70"
behaviors[0x1D78] = "horizontal_movement"
behaviors[0x1DA4] = "snow_ball"
behaviors[0x1DA8] = "rotating_cw_with_fire_bars"
behaviors[0x1DCC] = "lll_rotating_hex_flame"
behaviors[0x1E04] = "lll_wood_piece"
behaviors[0x1E30] = "floating_wood_bridge"
behaviors[0x1E4C] = "volcano_flames"
behaviors[0x1E6C] = "lll_rotating_platform"
behaviors[0x1E94] = "lll_slow_tilting_movement"
behaviors[0x1EC4] = "lll_slow_up_down_movement"
behaviors[0x1EF8] = "tilting_inverted_pyramid2"
behaviors[0x1F30] = "unused_1F30"
behaviors[0x1F3C] = "koopa_shell"
behaviors[0x1F68] = "koopa_shell_flame"
behaviors[0x1F90] = "tox_box"
behaviors[0x1FBC] = "piranha_plant_2"
behaviors[0x2018] = "lll_hexagonal_mesh"
behaviors[0x2038] = "bowser_puzzle_piece"
behaviors[0x2068] = "bowser_puzzle"
behaviors[0x2088] = "tuxies_mother"
behaviors[0x20D8] = "penguin_baby"
behaviors[0x20E0] = "unused_20E0"
behaviors[0x20E8] = "small_penguin"
behaviors[0x213C] = "fish_2"
behaviors[0x214C] = "fish_3"
behaviors[0x215C] = "large_fish_group"
behaviors[0x2160] = "fish_common"
behaviors[0x2178] = "fish_group2"
behaviors[0x2194] = "wdw_express_elevator"
behaviors[0x21C0] = "wdw_express_elevator_platform"
behaviors[0x21E4] = "chirp_chirp"
behaviors[0x220C] = "cheep_cheep"
behaviors[0x2250] = "exclamation_box"
behaviors[0x227C] = "rotating_exclamation_mark"
behaviors[0x229C] = "plays_sound"
behaviors[0x22B8] = "rock_solid"
behaviors[0x22D8] = "bowser_sub_door"
behaviors[0x2308] = "bowsers_sub"
behaviors[0x2338] = "sushi_shark"
behaviors[0x2388] = "sushi_shark_collision_child"
behaviors[0x23A4] = "jrb_sliding_box"
behaviors[0x23D0] = "ship_part_3"
behaviors[0x23EC] = "in_sunken_ship_3"
behaviors[0x241C] = "sunken_ship_part"
behaviors[0x243C] = "unused_243C"
behaviors[0x244C] = "sunken_ship_part_2"
behaviors[0x246C] = "in_sunken_ship"
behaviors[0x2480] = "in_sunken_ship_2"
behaviors[0x24AC] = "mario_dust_generator"
behaviors[0x24DC] = "white_puff1"
behaviors[0x2500] = "white_puff2"
behaviors[0x2528] = "white_puff_smoke2"
behaviors[0x2558] = "purple_switch_hidden_boxes"
behaviors[0x2568] = "blue_coin_switch"
behaviors[0x2588] = "hidden_blue_coin"
behaviors[0x25C0] = "bob_hmc_cage_door"
behaviors[0x25E0] = "openable_grill"
behaviors[0x25F8] = "water_level_trigger"
behaviors[0x2620] = "initialize_water_level_trigger"
behaviors[0x2634] = "tweester_sand_particle"
behaviors[0x2650] = "tweester"
behaviors[0x2684] = "spawn_big_boo"
behaviors[0x269C] = "animated_texture"
behaviors[0x26D4] = "boo_in_castle"
behaviors[0x2710] = "boo_with_cage"
behaviors[0x2768] = "patrolling_boo"
behaviors[0x277C] = "spawned_boo"
behaviors[0x2790] = "boo_giving_star"
behaviors[0x27D0] = "boo_group"
behaviors[0x27E4] = "boo_3"
behaviors[0x27F4] = "spawned_boo_2"
behaviors[0x2804] = "boo_2"
behaviors[0x286C] = "hidden_staircase_step"
behaviors[0x2898] = "boo_boss_spawned_bridge"
behaviors[0x28CC] = "bbh_tilt_floor_platforms"
behaviors[0x28FC] = "tumbling_bookshelf"
behaviors[0x292C] = "mesh_elevator"
behaviors[0x2968] = "rotating_merry_go_round"
behaviors[0x2998] = "plays_music_track_when_touched"
behaviors[0x29B0] = "inside_cannon"
behaviors[0x29B4] = "bait_coin"
behaviors[0x29E4] = "static_checkered_platform"
behaviors[0x2A10] = "unused_2A10"
behaviors[0x2A20] = "star"
behaviors[0x2A48] = "static_object"
behaviors[0x2A54] = "unused_2A54"
behaviors[0x2A5C] = "castle_floor_trap"
behaviors[0x2A7C] = "floor_trap_in_castle"
behaviors[0x2AA4] = "tree"
behaviors[0x2AD0] = "powerup_sparkles1"
behaviors[0x2AF0] = "powerup_sparkles2"
behaviors[0x2B08] = "special_triple_jump_sparkles"
behaviors[0x2B14] = "some_gfx"
behaviors[0x2B5C] = "scuttlebug"
behaviors[0x2BA0] = "scuttlebug_spawn"
behaviors[0x2BB8] = "whomp_king_boss"
behaviors[0x2BCC] = "small_whomp"
behaviors[0x2C14] = "water_splash"
behaviors[0x2C60] = "water_drops"
behaviors[0x2C7C] = "water_surface_white_wave"
behaviors[0x2CB0] = "gfx"
behaviors[0x2CE0] = "surface_waves"
behaviors[0x2D28] = "water_surface_white_wave_2"
behaviors[0x2D50] = "waves_generator"
behaviors[0x2D7C] = "surface_wave_shrinking"
behaviors[0x2DB0] = "water_type"
behaviors[0x2DC0] = "wave_trail_on_surface"
behaviors[0x2E04] = "tiny_white_wind_particle"
behaviors[0x2E20] = "wind_particle"
behaviors[0x2E3C] = "snowman_wind_blowing"
behaviors[0x2E58] = "walking_penguin"
behaviors[0x2EA8] = "yellow_ball"
behaviors[0x2EC0] = "mario"
behaviors[0x2EF8] = "toad_message"
behaviors[0x2F40] = "sealed_door_star"
behaviors[0x2F60] = "warps_60"
behaviors[0x2F64] = "warps_64"
behaviors[0x2F68] = "warps_68"
behaviors[0x2F6C] = "warps_6C"
behaviors[0x2F70] = "warps_70"
behaviors[0x2F74] = "warps_74"
behaviors[0x2F78] = "warps_78"
behaviors[0x2F7C] = "warps_7C"
behaviors[0x2F80] = "warps_80"
behaviors[0x2F84] = "warps_84"
behaviors[0x2F88] = "warps_88"
behaviors[0x2F8C] = "warps_8C"
behaviors[0x2F90] = "warps_90"
behaviors[0x2F94] = "warps_94"
behaviors[0x2FA0] = "random_animated_texture"
behaviors[0x2FC0] = "yellow_background_in_menu"
behaviors[0x2FE4] = "menu_growing_button"
behaviors[0x3008] = "gray_button"
behaviors[0x302C] = "star_in_act_selector"
behaviors[0x3048] = "act_selector"
behaviors[0x3068] = "coin_inside_of_formation"
behaviors[0x30A4] = "moving_blue_coin"
behaviors[0x30D4] = "blue_coin_sliding"
behaviors[0x3104] = "blue_coin_jumping"
behaviors[0x3134] = "seaweed"
behaviors[0x3158] = "seaweed_bundle"
behaviors[0x3174] = "bobomb"
behaviors[0x31AC] = "bobomb_dust"
behaviors[0x31DC] = "pink_bobomb_with_message"
behaviors[0x3228] = "pink_bobomb_opening_cannon"
behaviors[0x3274] = "cannon_trap_door"
behaviors[0x32A8] = "whirlpool_death"
behaviors[0x32C8] = "bubbles"
behaviors[0x32E0] = "message_panel"
behaviors[0x3324] = "sign_on_wall"
behaviors[0x3354] = "amp_2"
behaviors[0x3388] = "amp"
behaviors[0x33BC] = "butterfly"
behaviors[0x33EC] = "owl"
behaviors[0x3420] = "beta_green_shell"
behaviors[0x3454] = "carry_something1"
behaviors[0x345C] = "carry_something2"
behaviors[0x3464] = "carry_something3"
behaviors[0x346C] = "carry_something4"
behaviors[0x3474] = "carry_something5"
behaviors[0x347C] = "carry_something6"
behaviors[0x3484] = "dirt"
behaviors[0x34C4] = "water_wave"
behaviors[0x3510] = "explosion"
behaviors[0x3558] = "large_white_smoke_cloud"
behaviors[0x3588] = "smoke"
behaviors[0x35B0] = "white_particle_water_drop"
behaviors[0x3600] = "white_particle_water_drop_3600"
behaviors[0x3614] = "breakable_box_backup"
behaviors[0x362C] = "small_bully"
behaviors[0x3660] = "big_bully_boss"
behaviors[0x3694] = "bully"
behaviors[0x36C8] = "small_ice_bully"
behaviors[0x3700] = "chill_bully_boss"
behaviors[0x3738] = "manta_rings_generator"
behaviors[0x3750] = "air_bubble_water_rings"
behaviors[0x3798] = "air_bubble_water_rings_underwater"
behaviors[0x37E0] = "something_invisible"
behaviors[0x37EC] = "bowser_bomb"
behaviors[0x381C] = "spikey_bomb_explosion_1"
behaviors[0x3840] = "spikey_bomb_explosion_2"
behaviors[0x3868] = "star_celebration_star"
behaviors[0x3888] = "key_grab_sparkles"
behaviors[0x38B0] = "some_white_puff"
behaviors[0x38D0] = "bascule_bridge"
behaviors[0x38E8] = "lll_tilting_bridge"
behaviors[0x3910] = "rock_formation_eyes"
behaviors[0x3940] = "large_rock_eyes"
behaviors[0x3970] = "sliding_brick_platform_2"
behaviors[0x39A0] = "moneybag_transforming"
behaviors[0x39D4] = "moneybag"
behaviors[0x3A08] = "rolling_ball"
behaviors[0x3A30] = "rolling_metal_ball"
behaviors[0x3A58] = "tiny_metal_ball"
behaviors[0x3A80] = "metal_ball_generator_2"
behaviors[0x3AA4] = "metal_ball_generator"
behaviors[0x3AC8] = "mini_metal_balls_generator"
behaviors[0x3AE0] = "flapping_wing"
behaviors[0x3B00] = "spindel"
behaviors[0x3B30] = "moving_up_and_down"
behaviors[0x3B60] = "pyramid_elevator"
behaviors[0x3B98] = "pyramid_elevator_metal_balls"
behaviors[0x3BB4] = "pyramid_top"
behaviors[0x3BEC] = "pyramid_top_explosion"
behaviors[0x3C0C] = "collision_box_subbehavior"
behaviors[0x3C30] = "waterfall_sound_loop"
behaviors[0x3C44] = "volcano_sound_loop"
behaviors[0x3C58] = "castle_flag_waving"
behaviors[0x3C7C] = "birds_sound_loop"
behaviors[0x3C90] = "ambiant_sounds"
behaviors[0x3CA4] = "sand_sound_loop"
behaviors[0x3CB8] = "hidden_at_120_stars"
behaviors[0x3CE4] = "snowmans_bottom"
behaviors[0x3D0C] = "snowmans_head"
behaviors[0x3D34] = "snowmans_body_checkpoint"
behaviors[0x3D4C] = "big_snowman_whole"
behaviors[0x3D74] = "big_boulder"
behaviors[0x3DA0] = "big_boulder_generator"
behaviors[0x3DB8] = "wing_cap"
behaviors[0x3DD8] = "metal_cap"
behaviors[0x3DF8] = "normal_cap"
behaviors[0x3E1C] = "vanish_cap"
behaviors[0x3E3C] = "collect_star"
behaviors[0x3E64] = "star_spawn_coordinates"
behaviors[0x3E8C] = "hidden_red_coin_star"
behaviors[0x3EAC] = "red_coin"
behaviors[0x3EE4] = "bowser_course_red_coin_star"
behaviors[0x3EFC] = "hidden_star"
behaviors[0x3F1C] = "checkpoint"
behaviors[0x3F40] = "pitoune_2"
behaviors[0x3F78] = "falling_when_mario_is_near"
behaviors[0x3FA4] = "pitoune"
behaviors[0x3FDC] = "1up_walking"
behaviors[0x4010] = "1up_running_away"
behaviors[0x4044] = "1up_sliding"
behaviors[0x407C] = "1up"
behaviors[0x40B4] = "1up_jump_on_approach"
behaviors[0x40EC] = "1up_hidden"
behaviors[0x4124] = "1up_hidden_trigger"
behaviors[0x4148] = "1up_hidden_tree_pole"
behaviors[0x4180] = "1up_hidden_tree_pole_trigger"
behaviors[0x41A4] = "1up_hidden_in_tree"
behaviors[0x41BC] = "controllable_platform"
behaviors[0x41F0] = "controllable_platform_sub"
behaviors[0x4218] = "breakable_box2"
behaviors[0x4244] = "sliding_snow_mound"
behaviors[0x4270] = "snow_mound_spawn"
behaviors[0x4284] = "square_floating_platform"
behaviors[0x42B4] = "rect_floating_platform"
behaviors[0x42E4] = "jrb_floating_platform"
behaviors[0x4314] = "arrow_lift"
behaviors[0x4348] = "orange_number"
behaviors[0x4370] = "manta_ray"
behaviors[0x43A0] = "falling_pillar"
behaviors[0x43C4] = "some_subojbject_of_falling_pillar"
behaviors[0x43E0] = "pillar_base"
behaviors[0x4400] = "jrb_floating_box"
behaviors[0x442C] = "oscillating_pendulum"
behaviors[0x444C] = "treasure_chests_ship"
behaviors[0x4470] = "treasure_chests_jrb"
behaviors[0x4494] = "treasure_chests"
behaviors[0x44B8] = "treasure_chest_bottom"
behaviors[0x44E0] = "treasure_chest_top"
behaviors[0x44FC] = "mips"
behaviors[0x4538] = "yoshi"
behaviors[0x4580] = "koopa"
behaviors[0x45D0] = "waving_koopa_flag"
behaviors[0x45F8] = "koopa_flag"
behaviors[0x4634] = "pokey"
behaviors[0x4668] = "pokey_head"
behaviors[0x4698] = "bat"
behaviors[0x46DC] = "fly_guy"
behaviors[0x472C] = "goomba_walk"
behaviors[0x4770] = "goombas_group"
behaviors[0x478C] = "chain_chomp"
behaviors[0x47E4] = "chain_chomp_chains"
behaviors[0x481C] = "poundable"
behaviors[0x4868] = "chain_chomp_freed_explosion"
behaviors[0x4898] = "wiggler"
behaviors[0x48E0] = "wiggler_body_parts"
behaviors[0x4918] = "evil_lakitu"
behaviors[0x4954] = "lakitu"
behaviors[0x4988] = "fwoosh_blowing_wind"
behaviors[0x49AC] = "fwoosh_face"
behaviors[0x49C8] = "spiny"
behaviors[0x4A00] = "monty_mole"
behaviors[0x4A58] = "monty_mole_in_hole"
behaviors[0x4A78] = "monty_mole_rock"
behaviors[0x4AB0] = "platform_on_tracks"
behaviors[0x4AF4] = "metal_balls_for_elevators"
behaviors[0x4B1C] = "solid_moving"
behaviors[0x4B44] = "four_rotating_platforms"
behaviors[0x4B6C] = "ferris_wheel_platform"
behaviors[0x4B8C] = "water_bubble_drop"
behaviors[0x4BA8] = "water_bomb"
behaviors[0x4BD4] = "water_bomb_shadow"
behaviors[0x4BF0] = "rotating_clock_platform"
behaviors[0x4C24] = "clock_pendulum"
behaviors[0x4C5C] = "clock_metal_platform"
behaviors[0x4C94] = "sliding_platform"
behaviors[0x4CCC] = "rotating_gear_platform"
behaviors[0x4CF8] = "pushable_clock_box"
behaviors[0x4D28] = "clock_platform"
behaviors[0x4D64] = "clock_main_rotation"
behaviors[0x4D90] = "rotating_clock_platform2"
behaviors[0x4DBC] = "mr_blizzard"
behaviors[0x4E08] = "snowman_snowball"
behaviors[0x4E4C] = "sliding_platform2"
behaviors[0x4E78] = "octagonal_platform_rotating"
behaviors[0x4EA0] = "floor_switch_press_animation"
behaviors[0x4ECC] = "climbable_cage"
behaviors[0x4EF8] = "recover_life"
behaviors[0x4F10] = "cannon"
behaviors[0x4F28] = "cannon_barrel_bubbles"
behaviors[0x4F40] = "unagi"
behaviors[0x4F78] = "unagi_subobject"
behaviors[0x4F90] = "dorrie"
behaviors[0x4FD4] = "haunted_chair"
behaviors[0x5024] = "mad_piano"
behaviors[0x506C] = "flying_bookend"
behaviors[0x50B4] = "bookend_spawn"
behaviors[0x50D4] = "bookshelf_thing"
behaviors[0x50F4] = "book_switch"
behaviors[0x5120] = "piranha_plant"
behaviors[0x5158] = "small_piranha_flame"
behaviors[0x518C] = "fire_spewer"
behaviors[0x51AC] = "flyguy_flame"
behaviors[0x51E0] = "snufit"
behaviors[0x521C] = "snufit_balls"
behaviors[0x525C] = "grindel2"
behaviors[0x52B4] = "eyerok_boss"
behaviors[0x52D0] = "eyerok_hand"
behaviors[0x5310] = "klepto"
behaviors[0x5354] = "multiple_birds_flying"
behaviors[0x5380] = "racing_penguin"
behaviors[0x53C4] = "finish_line_check"
behaviors[0x53DC] = "shortcut_check"
behaviors[0x53F4] = "haunted_room_check"
behaviors[0x5414] = "haunted_room_check_subobject"
behaviors[0x5440] = "clam_shell"
behaviors[0x5468] = "skeeter"
behaviors[0x54A0] = "water_wave_type"
behaviors[0x54B8] = "pendulum_movement"
behaviors[0x54EC] = "donut_platform_group"
behaviors[0x5504] = "donut_platform"
behaviors[0x5528] = "ddd_yellow_poles"
behaviors[0x556C] = "shadow_star_for_red_star"
behaviors[0x5598] = "three_butterflies"
behaviors[0x55DC] = "bubba"
behaviors[0x5610] = "beginning_lakitu"
behaviors[0x5638] = "beginning_peach"
behaviors[0x565C] = "end_birds_1"
behaviors[0x5680] = "end_birds_2"
behaviors[0x56A4] = "intro_scene"

-- read float triple from memory
function read3float(base)
   arr = {}
   for i=0,2 do
      arr[i] = memory.readfloat(base + 4*i, true)
   end
   return arr
end

-- parse object data from address base
function ReadObject(base)
   local obj = {}
   obj.scale = read3float(base + 0x2C)
   obj.animFrame = memory.read_s16_be(base + 0x40)
   obj.active = memory.read_s32_be(base + 0x74)
   obj.position = read3float(base + 0xA0)
   obj.rotation = read3float(base + 0xD0)
   obj.rotation2 = read3float(base + 0xDC)
   obj.scaleXYZ = memory.readfloat(base + 0xF8, true)
   obj.action1 = memory.read_s32_be(base + 0x124)
   obj.action2 = memory.read_s32_be(base + 0x144)
   obj.currentAction = memory.read_s32_be(base + 0x14C)
   obj.action3 = memory.read_s32_be(base + 0x150)
   obj.timer = memory.read_s32_be(base + 0x154)
   obj.distance = memory.readfloat(base + 0x15C, true)
   obj.appear = memory.readfloat(base + 0x19C, true)
   obj.collision = {memory.readfloat(base + 0x1F8, true), memory.readfloat(base + 0x1FC, true)}
   obj.behavior = memory.read_u32_be(base + 0x20C)
   obj.vals = {}
   local i = 0
   -- read u32 of each value for hex display
   for off=0x000,0x260,0x4 do
      obj.vals[i] = memory.read_u32_be(base + off)
      i = i + 1
   end
   return obj
end

-- print float triple
function print3float(x, y, name, pos)
  gui.text(x, y, string.format("%s: %.1f, %.1f, %.1f", name, pos[2], pos[0], pos[1]))
end

local button = {}
local prev_button = {}

local obj_base = 0x033D488 -- base address of objects array
local obj_len = 0x260      -- length of each object in bytes
local cur_obj = 0          -- current object selected
local spacing = 14         -- spacing between each line
local hex_off = 0x000      -- current offset in hex display

while true do
   local obj_count = memory.read_u32_be(0x033D270)

   button = joypad.get(1)
   -- handle buttons
   if button["DPad L"] and not prev_button["DPad L"] then
      cur_obj = cur_obj - 1
      if cur_obj < 0 then
         cur_obj = obj_count - 1
      end
   end
   if button["DPad R"] and not prev_button["DPad R"] then
      cur_obj = cur_obj + 1
      if cur_obj >= obj_count then
         cur_obj = 0
      end
   end
   if button["DPad U"] and not prev_button["DPad U"] then
      hex_off = hex_off - 0x130
      if hex_off < 0 then
         hex_off = 0x260 -- off
      end
   end
   if button["DPad D"] and not prev_button["DPad D"] then
      hex_off = hex_off + 0x130
      if hex_off > 0x260 then
         hex_off = 0x000
      end
   end
   prev_button = button

   -- print object attributes
   local obj_addr = obj_base + obj_len * cur_obj
   local obj = ReadObject(obj_addr)
   local beh = obj.behavior - 0x800EB180
   if behaviors[beh] then
      gui.text(0,  0, string.format("Object %d %08X Type: %4X (%s)", cur_obj, 0x80000000 + obj_addr, beh, behaviors[beh]))
   else
      gui.text(0,  0, string.format("Object %d %08X Type: %4X (unknown)", cur_obj, 0x80000000 + obj_addr, beh))
   end
   if (obj.active > 0) then
      print3float(0, 1*spacing, "Scale", obj.scale)
      print3float(0, 2*spacing, "Pos", obj.position)
      print3float(0, 3*spacing, "Rot1", obj.rotation)
      print3float(0, 4*spacing, "Rot2", obj.rotation2)
      gui.text(0, 5*spacing, string.format("Dist: %.1f (appear %.1f)", obj.distance, obj.appear))
      gui.text(0, 6*spacing, string.format("Col Sphere: %.1f, %.1f", obj.collision[1], obj.collision[2]))
   else
      gui.text(0, 1*spacing, "Inactive")
   end

   -- print hex values
   if hex_off < 0x260 then
      local idx = hex_off/4
      for y=0,0x130/16-1 do
         gui.text(0, (14+y)*spacing, string.format("%03X: %08X %08X %08X %08X", hex_off + y*16, obj.vals[idx+y*4], obj.vals[idx+y*4+1], obj.vals[idx+y*4+2], obj.vals[idx+y*4+3]))
      end
   end

   gui.text(0, 464, "Objects " .. obj_count)
   emu.frameadvance()
end

